//******************************************************************************
//******************************************************************************
/**
 * @file    IBPNGClientListener.h
 * @author  Markus van Pinxteren
 * @date    12.05.2010
 *
 * @brief   Interface class for the BPNGClient listener
*/
//******************************************************************************
//******************************************************************************

#ifndef __IBPNGCLIENTLISTENER_HH__
#define __IBPNGCLIENTLISTENER_HH__

#ifdef _WINDOWS
#include <windows.h>
#else
#include <iostream>
#endif
#include "BPNGDefines.h"

extern "C" 
{
    struct IBPNGClientListener
    {
        /// Called to notify a detected logger in network
        /**
          All char* of the passed OnlineLoggerInfo* are only valid for the
          time of the function call. Please ensure to copy the string values.
          */
        virtual void WINAPI onBPNGDeviceDetected(OnlineLoggerInfo* info) = 0;

        /// Called to notify a disappeared logger
        /**
          All char* of the passed OnlineLoggerInfo* are only valid for the
          time of the function call. Please ensure to copy the string values.
          */
        virtual void WINAPI onBPNGDeviceDisappeared(OnlineLoggerInfo* info) = 0;

        /// Called to notify a logger's state change
        /**
          All char* of the passed OnlineLoggerInfo* are only valid for the
          time of the function call. Please ensure to copy the string values.
          */
        virtual void WINAPI onBPNGDeviceStateChange(OnlineLoggerInfo* info) = 0;

        /// Called to indicate the current progress of a file transfer
        /**
          This function notifies the listener about the download progress of the raw Telemotive trace data.

          @param percentCompleted Percentage of the entire download process (from 0...100%).
          A negative value can be passed if only the abort request is checked.
          A negative value of -1 indicates a broken ftp connection.
          @return return value 0 indicates an abort request from the implementing class
          */
        virtual int WINAPI onProgressDataDownload(int percentCompleted) = 0;

        /// Called to indicate the current progress of file conversion
        /**
          This function notifies the listener about the conversion progress of the raw Telemotive trace data.
          If the \a percentCompleted value has changed, but the \a status is still the
          same, the application passes an empty string as status to the function.

          @param percentCompleted Percent of the entire conversion process (from 0...100%), -1 indicates the same value as from last function call
          @param status Status of the conversion process (e.g. "Converting trace data. Block 5 of 32")
          @return return value 0 indicates an abort request from the implementing class
          */
        virtual int WINAPI onProgressConversion(int percentCompleted, const char* status) = 0;

        /// Called to send additional information of the current process to the calling app
        /**
          This function transmit message strings to the listener class. Those messages are only
          for imformation purpose. The receiver doesn't have to react on it but can display it
          on the screen.
          */
        virtual void WINAPI onStatusMessage(const char* statusMsg) = 0;

        /// Called to send additional information of the current data recovery progress
        /**
        This function transmit message informations for the data recovery process. Those messages are only for information purpose.
        The information contains a String information about the current data recovery process and int value which contains a percent value for progressbar
        */
        virtual int WINAPI onDataRecoverProgress(const char* statusMsg, int percentage) = 0;

        /// Called to inform about a warning.
        /**
          This function transmit a warning message to the listener class. Warnings have a WARING_CODE and a warning message.
          Warnings do not interrupt the current process but should be noticed from the user to possibly initiate further provisions.
          */
        virtual void WINAPI onWarning(BPNGWarningCode warningCode, const char* warnMsg) = 0;

        /// Called on a too long target directory
        /**
          Called when the resulting file name of the converted files or the files of an offline data set is longer than the maximum
          allowed size of the file system (Windows 260). The lib user has to pass a new (shorter) base target directory to the passed
          char array with strcpy. The memory of the array is already allocated by the library and it's size is maxSize.
          When a new directory was set the value 1 must be returned. Returning another value than 1 will abort the current process with
          an error result.
          */
        virtual int WINAPI onTargetPathTooLong(char* newTarget, int maxSize) = 0;

        /// Called on existing output trace files
        /**
          When an output trace file already exists this function is called. The listener has the possibility to
          return one of following values:
          -1: no, don't overwrite file
          -2: no, overwrite neither this nor any following file
          1: yes, overwrite file
          2: yes, overwrite this and all following files
          0: cancel conversion
          */
        virtual int WINAPI getOverwritingPermission(const char* filePath) = 0;


        /// Called on accessing password protected functions
        /**
         When password protected functions are called this listener function queries
         for login parameters that must be returned from the implementing class.

         @param ipAddress IP address of the password protected device
         */
        virtual const char* WINAPI onLogInDataRequired(unsigned mbnr) = 0;

        /// Called if invalid pw file found on device
        /**
          An error may occured on transfering the passwordconfiguration to the device, as a result the passwordconfiguration is invalid
          and needs to be reset to default. Inform the user.
          */
        virtual void WINAPI onInvalidPwConfigFound(unsigned mbnr) = 0;

        //
        virtual void WINAPI onLogInDataFailed() = 0;
        //
        virtual void WINAPI onResetLogInDataFailed() = 0;

        //
        virtual void WINAPI onFuncAccessDenied() = 0;

        /// Called in case of not enough free diskspace
        /**
            This notifies the listener about not enough free disk space for data download or conversion.
            The user can continue or abort the process. Returning 0 will abort the process.
            In some cases continuing without providing more disk space will call this function
            immediately again.
            @param freeSpace Amount of free space
            @param neededSpace Amount of needed space
            @param drive Name of the drive where to store data
            @param msg Additional message to display
            @return return 0 when process should be aborted, 1 to ignore
            */
        virtual int WINAPI onCriticalDiskSpace(uint64_t freeSpace, uint64_t neededSpace, const char* drive, const char* msg) = 0;

        /// Called on firmware update progress
        virtual void WINAPI onFirmwareUpdateProgress(int percentage, int stepId, int subStepId, const char* desc) = 0;
        virtual void WINAPI onFirmwareUpdateError(int  errorId) = 0;

        /// Called on creation of log report
        /// @return return value 0 indicates an abort request from the implementing class
        virtual int WINAPI onGetLogReportProgress(int percentage, const char* desc) = 0;

        /// Notifies the listeners before the download starts about the total amount of bytes to be downloaded
        /**
        @param totalAmountOfBytes Total data size to be downloaded

        */
        virtual void WINAPI onDownloadStart(int64_t totalAmountOfBytes) = 0;

        /// Notifies the listeners before the conversion starts about the total amount of bytes to be converted
        /**
        @param totalAmountOfBytes Total data size to be converted

        */
        virtual void WINAPI onConversionStart(int64_t totalAmountOfBytes) = 0;

        /// Notifies the listeners that a password for an archive extraction is required, this will be called on EVERY archive that needs a password
        /// nethertheless a password was already entered. Already entered passwords should be handled by the callbacked instance.
        /**
        @param retryCount number of attempty on one file, on zero its first try
        The callbacked instance can save a password list and try every password on the list, if retryCount is zero the list should be handled from the
        start. If no password is left return 0.
        */
        virtual const char* WINAPI onExtractionPasswordRequired(uint8_t retryCount) = 0;
    };

} // extern C

#endif // __IBPNGCLIENTLISTENER_HH__
